%% Vendored from hex_core v0.8.2, do not edit manually

%% File generated by spdx.ex. Do not edit manually.

-module(mix_hex_licenses).

-export([valid/1]).

valid(<<"0BSD">>) -> true;
valid(<<"AAL">>) -> true;
valid(<<"ADSL">>) -> true;
valid(<<"AFL-1.1">>) -> true;
valid(<<"AFL-1.2">>) -> true;
valid(<<"AFL-2.0">>) -> true;
valid(<<"AFL-2.1">>) -> true;
valid(<<"AFL-3.0">>) -> true;
valid(<<"AGPL-1.0">>) -> true;
valid(<<"AGPL-1.0-only">>) -> true;
valid(<<"AGPL-1.0-or-later">>) -> true;
valid(<<"AGPL-3.0">>) -> true;
valid(<<"AGPL-3.0-only">>) -> true;
valid(<<"AGPL-3.0-or-later">>) -> true;
valid(<<"AMDPLPA">>) -> true;
valid(<<"AML">>) -> true;
valid(<<"AMPAS">>) -> true;
valid(<<"ANTLR-PD">>) -> true;
valid(<<"ANTLR-PD-fallback">>) -> true;
valid(<<"APAFML">>) -> true;
valid(<<"APL-1.0">>) -> true;
valid(<<"APSL-1.0">>) -> true;
valid(<<"APSL-1.1">>) -> true;
valid(<<"APSL-1.2">>) -> true;
valid(<<"APSL-2.0">>) -> true;
valid(<<"Abstyles">>) -> true;
valid(<<"Adobe-2006">>) -> true;
valid(<<"Adobe-Glyph">>) -> true;
valid(<<"Afmparse">>) -> true;
valid(<<"Aladdin">>) -> true;
valid(<<"Apache-1.0">>) -> true;
valid(<<"Apache-1.1">>) -> true;
valid(<<"Apache-2.0">>) -> true;
valid(<<"Artistic-1.0">>) -> true;
valid(<<"Artistic-1.0-Perl">>) -> true;
valid(<<"Artistic-1.0-cl8">>) -> true;
valid(<<"Artistic-2.0">>) -> true;
valid(<<"BSD-1-Clause">>) -> true;
valid(<<"BSD-2-Clause">>) -> true;
valid(<<"BSD-2-Clause-FreeBSD">>) -> true;
valid(<<"BSD-2-Clause-NetBSD">>) -> true;
valid(<<"BSD-2-Clause-Patent">>) -> true;
valid(<<"BSD-2-Clause-Views">>) -> true;
valid(<<"BSD-3-Clause">>) -> true;
valid(<<"BSD-3-Clause-Attribution">>) -> true;
valid(<<"BSD-3-Clause-Clear">>) -> true;
valid(<<"BSD-3-Clause-LBNL">>) -> true;
valid(<<"BSD-3-Clause-Modification">>) -> true;
valid(<<"BSD-3-Clause-No-Military-License">>) -> true;
valid(<<"BSD-3-Clause-No-Nuclear-License">>) -> true;
valid(<<"BSD-3-Clause-No-Nuclear-License-2014">>) -> true;
valid(<<"BSD-3-Clause-No-Nuclear-Warranty">>) -> true;
valid(<<"BSD-3-Clause-Open-MPI">>) -> true;
valid(<<"BSD-4-Clause">>) -> true;
valid(<<"BSD-4-Clause-Shortened">>) -> true;
valid(<<"BSD-4-Clause-UC">>) -> true;
valid(<<"BSD-Protection">>) -> true;
valid(<<"BSD-Source-Code">>) -> true;
valid(<<"BSL-1.0">>) -> true;
valid(<<"BUSL-1.1">>) -> true;
valid(<<"Bahyph">>) -> true;
valid(<<"Barr">>) -> true;
valid(<<"Beerware">>) -> true;
valid(<<"BitTorrent-1.0">>) -> true;
valid(<<"BitTorrent-1.1">>) -> true;
valid(<<"BlueOak-1.0.0">>) -> true;
valid(<<"Borceux">>) -> true;
valid(<<"C-UDA-1.0">>) -> true;
valid(<<"CAL-1.0">>) -> true;
valid(<<"CAL-1.0-Combined-Work-Exception">>) -> true;
valid(<<"CATOSL-1.1">>) -> true;
valid(<<"CC-BY-1.0">>) -> true;
valid(<<"CC-BY-2.0">>) -> true;
valid(<<"CC-BY-2.5">>) -> true;
valid(<<"CC-BY-3.0">>) -> true;
valid(<<"CC-BY-3.0-AT">>) -> true;
valid(<<"CC-BY-3.0-US">>) -> true;
valid(<<"CC-BY-4.0">>) -> true;
valid(<<"CC-BY-NC-1.0">>) -> true;
valid(<<"CC-BY-NC-2.0">>) -> true;
valid(<<"CC-BY-NC-2.5">>) -> true;
valid(<<"CC-BY-NC-3.0">>) -> true;
valid(<<"CC-BY-NC-4.0">>) -> true;
valid(<<"CC-BY-NC-ND-1.0">>) -> true;
valid(<<"CC-BY-NC-ND-2.0">>) -> true;
valid(<<"CC-BY-NC-ND-2.5">>) -> true;
valid(<<"CC-BY-NC-ND-3.0">>) -> true;
valid(<<"CC-BY-NC-ND-3.0-IGO">>) -> true;
valid(<<"CC-BY-NC-ND-4.0">>) -> true;
valid(<<"CC-BY-NC-SA-1.0">>) -> true;
valid(<<"CC-BY-NC-SA-2.0">>) -> true;
valid(<<"CC-BY-NC-SA-2.5">>) -> true;
valid(<<"CC-BY-NC-SA-3.0">>) -> true;
valid(<<"CC-BY-NC-SA-4.0">>) -> true;
valid(<<"CC-BY-ND-1.0">>) -> true;
valid(<<"CC-BY-ND-2.0">>) -> true;
valid(<<"CC-BY-ND-2.5">>) -> true;
valid(<<"CC-BY-ND-3.0">>) -> true;
valid(<<"CC-BY-ND-4.0">>) -> true;
valid(<<"CC-BY-SA-1.0">>) -> true;
valid(<<"CC-BY-SA-2.0">>) -> true;
valid(<<"CC-BY-SA-2.0-UK">>) -> true;
valid(<<"CC-BY-SA-2.1-JP">>) -> true;
valid(<<"CC-BY-SA-2.5">>) -> true;
valid(<<"CC-BY-SA-3.0">>) -> true;
valid(<<"CC-BY-SA-3.0-AT">>) -> true;
valid(<<"CC-BY-SA-4.0">>) -> true;
valid(<<"CC-PDDC">>) -> true;
valid(<<"CC0-1.0">>) -> true;
valid(<<"CDDL-1.0">>) -> true;
valid(<<"CDDL-1.1">>) -> true;
valid(<<"CDL-1.0">>) -> true;
valid(<<"CDLA-Permissive-1.0">>) -> true;
valid(<<"CDLA-Sharing-1.0">>) -> true;
valid(<<"CECILL-1.0">>) -> true;
valid(<<"CECILL-1.1">>) -> true;
valid(<<"CECILL-2.0">>) -> true;
valid(<<"CECILL-2.1">>) -> true;
valid(<<"CECILL-B">>) -> true;
valid(<<"CECILL-C">>) -> true;
valid(<<"CERN-OHL-1.1">>) -> true;
valid(<<"CERN-OHL-1.2">>) -> true;
valid(<<"CERN-OHL-P-2.0">>) -> true;
valid(<<"CERN-OHL-S-2.0">>) -> true;
valid(<<"CERN-OHL-W-2.0">>) -> true;
valid(<<"CNRI-Jython">>) -> true;
valid(<<"CNRI-Python">>) -> true;
valid(<<"CNRI-Python-GPL-Compatible">>) -> true;
valid(<<"CPAL-1.0">>) -> true;
valid(<<"CPL-1.0">>) -> true;
valid(<<"CPOL-1.02">>) -> true;
valid(<<"CUA-OPL-1.0">>) -> true;
valid(<<"Caldera">>) -> true;
valid(<<"ClArtistic">>) -> true;
valid(<<"Condor-1.1">>) -> true;
valid(<<"Crossword">>) -> true;
valid(<<"CrystalStacker">>) -> true;
valid(<<"Cube">>) -> true;
valid(<<"D-FSL-1.0">>) -> true;
valid(<<"DOC">>) -> true;
valid(<<"DRL-1.0">>) -> true;
valid(<<"DSDP">>) -> true;
valid(<<"Dotseqn">>) -> true;
valid(<<"ECL-1.0">>) -> true;
valid(<<"ECL-2.0">>) -> true;
valid(<<"EFL-1.0">>) -> true;
valid(<<"EFL-2.0">>) -> true;
valid(<<"EPICS">>) -> true;
valid(<<"EPL-1.0">>) -> true;
valid(<<"EPL-2.0">>) -> true;
valid(<<"EUDatagrid">>) -> true;
valid(<<"EUPL-1.0">>) -> true;
valid(<<"EUPL-1.1">>) -> true;
valid(<<"EUPL-1.2">>) -> true;
valid(<<"Entessa">>) -> true;
valid(<<"ErlPL-1.1">>) -> true;
valid(<<"Eurosym">>) -> true;
valid(<<"FSFAP">>) -> true;
valid(<<"FSFUL">>) -> true;
valid(<<"FSFULLR">>) -> true;
valid(<<"FTL">>) -> true;
valid(<<"Fair">>) -> true;
valid(<<"Frameworx-1.0">>) -> true;
valid(<<"FreeBSD-DOC">>) -> true;
valid(<<"FreeImage">>) -> true;
valid(<<"GD">>) -> true;
valid(<<"GFDL-1.1">>) -> true;
valid(<<"GFDL-1.1-invariants-only">>) -> true;
valid(<<"GFDL-1.1-invariants-or-later">>) -> true;
valid(<<"GFDL-1.1-no-invariants-only">>) -> true;
valid(<<"GFDL-1.1-no-invariants-or-later">>) -> true;
valid(<<"GFDL-1.1-only">>) -> true;
valid(<<"GFDL-1.1-or-later">>) -> true;
valid(<<"GFDL-1.2">>) -> true;
valid(<<"GFDL-1.2-invariants-only">>) -> true;
valid(<<"GFDL-1.2-invariants-or-later">>) -> true;
valid(<<"GFDL-1.2-no-invariants-only">>) -> true;
valid(<<"GFDL-1.2-no-invariants-or-later">>) -> true;
valid(<<"GFDL-1.2-only">>) -> true;
valid(<<"GFDL-1.2-or-later">>) -> true;
valid(<<"GFDL-1.3">>) -> true;
valid(<<"GFDL-1.3-invariants-only">>) -> true;
valid(<<"GFDL-1.3-invariants-or-later">>) -> true;
valid(<<"GFDL-1.3-no-invariants-only">>) -> true;
valid(<<"GFDL-1.3-no-invariants-or-later">>) -> true;
valid(<<"GFDL-1.3-only">>) -> true;
valid(<<"GFDL-1.3-or-later">>) -> true;
valid(<<"GL2PS">>) -> true;
valid(<<"GLWTPL">>) -> true;
valid(<<"GPL-1.0">>) -> true;
valid(<<"GPL-1.0+">>) -> true;
valid(<<"GPL-1.0-only">>) -> true;
valid(<<"GPL-1.0-or-later">>) -> true;
valid(<<"GPL-2.0">>) -> true;
valid(<<"GPL-2.0+">>) -> true;
valid(<<"GPL-2.0-only">>) -> true;
valid(<<"GPL-2.0-or-later">>) -> true;
valid(<<"GPL-2.0-with-GCC-exception">>) -> true;
valid(<<"GPL-2.0-with-autoconf-exception">>) -> true;
valid(<<"GPL-2.0-with-bison-exception">>) -> true;
valid(<<"GPL-2.0-with-classpath-exception">>) -> true;
valid(<<"GPL-2.0-with-font-exception">>) -> true;
valid(<<"GPL-3.0">>) -> true;
valid(<<"GPL-3.0+">>) -> true;
valid(<<"GPL-3.0-only">>) -> true;
valid(<<"GPL-3.0-or-later">>) -> true;
valid(<<"GPL-3.0-with-GCC-exception">>) -> true;
valid(<<"GPL-3.0-with-autoconf-exception">>) -> true;
valid(<<"Giftware">>) -> true;
valid(<<"Glide">>) -> true;
valid(<<"Glulxe">>) -> true;
valid(<<"HPND">>) -> true;
valid(<<"HPND-sell-variant">>) -> true;
valid(<<"HTMLTIDY">>) -> true;
valid(<<"HaskellReport">>) -> true;
valid(<<"Hippocratic-2.1">>) -> true;
valid(<<"IBM-pibs">>) -> true;
valid(<<"ICU">>) -> true;
valid(<<"IJG">>) -> true;
valid(<<"IPA">>) -> true;
valid(<<"IPL-1.0">>) -> true;
valid(<<"ISC">>) -> true;
valid(<<"ImageMagick">>) -> true;
valid(<<"Imlib2">>) -> true;
valid(<<"Info-ZIP">>) -> true;
valid(<<"Intel">>) -> true;
valid(<<"Intel-ACPI">>) -> true;
valid(<<"Interbase-1.0">>) -> true;
valid(<<"JPNIC">>) -> true;
valid(<<"JSON">>) -> true;
valid(<<"JasPer-2.0">>) -> true;
valid(<<"LAL-1.2">>) -> true;
valid(<<"LAL-1.3">>) -> true;
valid(<<"LGPL-2.0">>) -> true;
valid(<<"LGPL-2.0+">>) -> true;
valid(<<"LGPL-2.0-only">>) -> true;
valid(<<"LGPL-2.0-or-later">>) -> true;
valid(<<"LGPL-2.1">>) -> true;
valid(<<"LGPL-2.1+">>) -> true;
valid(<<"LGPL-2.1-only">>) -> true;
valid(<<"LGPL-2.1-or-later">>) -> true;
valid(<<"LGPL-3.0">>) -> true;
valid(<<"LGPL-3.0+">>) -> true;
valid(<<"LGPL-3.0-only">>) -> true;
valid(<<"LGPL-3.0-or-later">>) -> true;
valid(<<"LGPLLR">>) -> true;
valid(<<"LPL-1.0">>) -> true;
valid(<<"LPL-1.02">>) -> true;
valid(<<"LPPL-1.0">>) -> true;
valid(<<"LPPL-1.1">>) -> true;
valid(<<"LPPL-1.2">>) -> true;
valid(<<"LPPL-1.3a">>) -> true;
valid(<<"LPPL-1.3c">>) -> true;
valid(<<"Latex2e">>) -> true;
valid(<<"Leptonica">>) -> true;
valid(<<"LiLiQ-P-1.1">>) -> true;
valid(<<"LiLiQ-R-1.1">>) -> true;
valid(<<"LiLiQ-Rplus-1.1">>) -> true;
valid(<<"Libpng">>) -> true;
valid(<<"Linux-OpenIB">>) -> true;
valid(<<"MIT">>) -> true;
valid(<<"MIT-0">>) -> true;
valid(<<"MIT-CMU">>) -> true;
valid(<<"MIT-Modern-Variant">>) -> true;
valid(<<"MIT-advertising">>) -> true;
valid(<<"MIT-enna">>) -> true;
valid(<<"MIT-feh">>) -> true;
valid(<<"MIT-open-group">>) -> true;
valid(<<"MITNFA">>) -> true;
valid(<<"MPL-1.0">>) -> true;
valid(<<"MPL-1.1">>) -> true;
valid(<<"MPL-2.0">>) -> true;
valid(<<"MPL-2.0-no-copyleft-exception">>) -> true;
valid(<<"MS-PL">>) -> true;
valid(<<"MS-RL">>) -> true;
valid(<<"MTLL">>) -> true;
valid(<<"MakeIndex">>) -> true;
valid(<<"MirOS">>) -> true;
valid(<<"Motosoto">>) -> true;
valid(<<"MulanPSL-1.0">>) -> true;
valid(<<"MulanPSL-2.0">>) -> true;
valid(<<"Multics">>) -> true;
valid(<<"Mup">>) -> true;
valid(<<"NAIST-2003">>) -> true;
valid(<<"NASA-1.3">>) -> true;
valid(<<"NBPL-1.0">>) -> true;
valid(<<"NCGL-UK-2.0">>) -> true;
valid(<<"NCSA">>) -> true;
valid(<<"NGPL">>) -> true;
valid(<<"NIST-PD">>) -> true;
valid(<<"NIST-PD-fallback">>) -> true;
valid(<<"NLOD-1.0">>) -> true;
valid(<<"NLPL">>) -> true;
valid(<<"NOSL">>) -> true;
valid(<<"NPL-1.0">>) -> true;
valid(<<"NPL-1.1">>) -> true;
valid(<<"NPOSL-3.0">>) -> true;
valid(<<"NRL">>) -> true;
valid(<<"NTP">>) -> true;
valid(<<"NTP-0">>) -> true;
valid(<<"Naumen">>) -> true;
valid(<<"Net-SNMP">>) -> true;
valid(<<"NetCDF">>) -> true;
valid(<<"Newsletr">>) -> true;
valid(<<"Nokia">>) -> true;
valid(<<"Noweb">>) -> true;
valid(<<"Nunit">>) -> true;
valid(<<"O-UDA-1.0">>) -> true;
valid(<<"OCCT-PL">>) -> true;
valid(<<"OCLC-2.0">>) -> true;
valid(<<"ODC-By-1.0">>) -> true;
valid(<<"ODbL-1.0">>) -> true;
valid(<<"OFL-1.0">>) -> true;
valid(<<"OFL-1.0-RFN">>) -> true;
valid(<<"OFL-1.0-no-RFN">>) -> true;
valid(<<"OFL-1.1">>) -> true;
valid(<<"OFL-1.1-RFN">>) -> true;
valid(<<"OFL-1.1-no-RFN">>) -> true;
valid(<<"OGC-1.0">>) -> true;
valid(<<"OGDL-Taiwan-1.0">>) -> true;
valid(<<"OGL-Canada-2.0">>) -> true;
valid(<<"OGL-UK-1.0">>) -> true;
valid(<<"OGL-UK-2.0">>) -> true;
valid(<<"OGL-UK-3.0">>) -> true;
valid(<<"OGTSL">>) -> true;
valid(<<"OLDAP-1.1">>) -> true;
valid(<<"OLDAP-1.2">>) -> true;
valid(<<"OLDAP-1.3">>) -> true;
valid(<<"OLDAP-1.4">>) -> true;
valid(<<"OLDAP-2.0">>) -> true;
valid(<<"OLDAP-2.0.1">>) -> true;
valid(<<"OLDAP-2.1">>) -> true;
valid(<<"OLDAP-2.2">>) -> true;
valid(<<"OLDAP-2.2.1">>) -> true;
valid(<<"OLDAP-2.2.2">>) -> true;
valid(<<"OLDAP-2.3">>) -> true;
valid(<<"OLDAP-2.4">>) -> true;
valid(<<"OLDAP-2.5">>) -> true;
valid(<<"OLDAP-2.6">>) -> true;
valid(<<"OLDAP-2.7">>) -> true;
valid(<<"OLDAP-2.8">>) -> true;
valid(<<"OML">>) -> true;
valid(<<"OPL-1.0">>) -> true;
valid(<<"OSET-PL-2.1">>) -> true;
valid(<<"OSL-1.0">>) -> true;
valid(<<"OSL-1.1">>) -> true;
valid(<<"OSL-2.0">>) -> true;
valid(<<"OSL-2.1">>) -> true;
valid(<<"OSL-3.0">>) -> true;
valid(<<"OpenSSL">>) -> true;
valid(<<"PDDL-1.0">>) -> true;
valid(<<"PHP-3.0">>) -> true;
valid(<<"PHP-3.01">>) -> true;
valid(<<"PSF-2.0">>) -> true;
valid(<<"Parity-6.0.0">>) -> true;
valid(<<"Parity-7.0.0">>) -> true;
valid(<<"Plexus">>) -> true;
valid(<<"PolyForm-Noncommercial-1.0.0">>) -> true;
valid(<<"PolyForm-Small-Business-1.0.0">>) -> true;
valid(<<"PostgreSQL">>) -> true;
valid(<<"Python-2.0">>) -> true;
valid(<<"QPL-1.0">>) -> true;
valid(<<"Qhull">>) -> true;
valid(<<"RHeCos-1.1">>) -> true;
valid(<<"RPL-1.1">>) -> true;
valid(<<"RPL-1.5">>) -> true;
valid(<<"RPSL-1.0">>) -> true;
valid(<<"RSA-MD">>) -> true;
valid(<<"RSCPL">>) -> true;
valid(<<"Rdisc">>) -> true;
valid(<<"Ruby">>) -> true;
valid(<<"SAX-PD">>) -> true;
valid(<<"SCEA">>) -> true;
valid(<<"SGI-B-1.0">>) -> true;
valid(<<"SGI-B-1.1">>) -> true;
valid(<<"SGI-B-2.0">>) -> true;
valid(<<"SHL-0.5">>) -> true;
valid(<<"SHL-0.51">>) -> true;
valid(<<"SISSL">>) -> true;
valid(<<"SISSL-1.2">>) -> true;
valid(<<"SMLNJ">>) -> true;
valid(<<"SMPPL">>) -> true;
valid(<<"SNIA">>) -> true;
valid(<<"SPL-1.0">>) -> true;
valid(<<"SSH-OpenSSH">>) -> true;
valid(<<"SSH-short">>) -> true;
valid(<<"SSPL-1.0">>) -> true;
valid(<<"SWL">>) -> true;
valid(<<"Saxpath">>) -> true;
valid(<<"Sendmail">>) -> true;
valid(<<"Sendmail-8.23">>) -> true;
valid(<<"SimPL-2.0">>) -> true;
valid(<<"Sleepycat">>) -> true;
valid(<<"Spencer-86">>) -> true;
valid(<<"Spencer-94">>) -> true;
valid(<<"Spencer-99">>) -> true;
valid(<<"StandardML-NJ">>) -> true;
valid(<<"SugarCRM-1.1.3">>) -> true;
valid(<<"TAPR-OHL-1.0">>) -> true;
valid(<<"TCL">>) -> true;
valid(<<"TCP-wrappers">>) -> true;
valid(<<"TMate">>) -> true;
valid(<<"TORQUE-1.1">>) -> true;
valid(<<"TOSL">>) -> true;
valid(<<"TU-Berlin-1.0">>) -> true;
valid(<<"TU-Berlin-2.0">>) -> true;
valid(<<"UCL-1.0">>) -> true;
valid(<<"UPL-1.0">>) -> true;
valid(<<"Unicode-DFS-2015">>) -> true;
valid(<<"Unicode-DFS-2016">>) -> true;
valid(<<"Unicode-TOU">>) -> true;
valid(<<"Unlicense">>) -> true;
valid(<<"VOSTROM">>) -> true;
valid(<<"VSL-1.0">>) -> true;
valid(<<"Vim">>) -> true;
valid(<<"W3C">>) -> true;
valid(<<"W3C-19980720">>) -> true;
valid(<<"W3C-20150513">>) -> true;
valid(<<"WTFPL">>) -> true;
valid(<<"Watcom-1.0">>) -> true;
valid(<<"Wsuipa">>) -> true;
valid(<<"X11">>) -> true;
valid(<<"XFree86-1.1">>) -> true;
valid(<<"XSkat">>) -> true;
valid(<<"Xerox">>) -> true;
valid(<<"Xnet">>) -> true;
valid(<<"YPL-1.0">>) -> true;
valid(<<"YPL-1.1">>) -> true;
valid(<<"ZPL-1.1">>) -> true;
valid(<<"ZPL-2.0">>) -> true;
valid(<<"ZPL-2.1">>) -> true;
valid(<<"Zed">>) -> true;
valid(<<"Zend-2.0">>) -> true;
valid(<<"Zimbra-1.3">>) -> true;
valid(<<"Zimbra-1.4">>) -> true;
valid(<<"Zlib">>) -> true;
valid(<<"blessing">>) -> true;
valid(<<"bzip2-1.0.5">>) -> true;
valid(<<"bzip2-1.0.6">>) -> true;
valid(<<"copyleft-next-0.3.0">>) -> true;
valid(<<"copyleft-next-0.3.1">>) -> true;
valid(<<"curl">>) -> true;
valid(<<"diffmark">>) -> true;
valid(<<"dvipdfm">>) -> true;
valid(<<"eCos-2.0">>) -> true;
valid(<<"eGenix">>) -> true;
valid(<<"etalab-2.0">>) -> true;
valid(<<"gSOAP-1.3b">>) -> true;
valid(<<"gnuplot">>) -> true;
valid(<<"iMatix">>) -> true;
valid(<<"libpng-2.0">>) -> true;
valid(<<"libselinux-1.0">>) -> true;
valid(<<"libtiff">>) -> true;
valid(<<"mpich2">>) -> true;
valid(<<"psfrag">>) -> true;
valid(<<"psutils">>) -> true;
valid(<<"wxWindows">>) -> true;
valid(<<"xinetd">>) -> true;
valid(<<"xpp">>) -> true;
valid(<<"zlib-acknowledgement">>) -> true;
valid(_) -> false.
